@@ -34,7 +34,7 @@ from openiti.helper.templates import author_yml_template, book_yml_template, \
 
 
 
-def ymlToDic(yml_str, reflow=True):
+def ymlToDic(yml_str, reflow=False, yml_fp=""):
     """Convert a yml string into a dictionary.
 
     NB: in order to be read correctly, OpenITI yml keys (lemmata) should always:
@@ -75,7 +75,7 @@ def ymlToDic(yml_str, reflow=True):
         
     """
     if yml_str.strip() == "":
-        raise Exception("Empty YML file!")
+        raise Exception(yml_fp, "Empty YML file!")
         return {}
     
     # normalize new line characters:
@@ -101,21 +101,28 @@ def ymlToDic(yml_str, reflow=True):
     data = data.split("\n")
     dic = dict()
     for d in data:
-        spl = re.split(r"(^[\w#]+:)", d, 1)
+        spl = re.split(r"(^[\w#]+:+)", d, 1)
         try:
             dic[spl[1]] = spl[2].strip()
         except:
-            raise Exception("no valid yml key in line", d)
+            raise Exception(yml_fp, "no valid yml key in line", d)
 
     return dic
 
 
-def readYML(fp):
+def readYML(fp, reflow=False):
     """Read a yml file and convert it into a dictionary.
 
     Args:
         fp (str): path to the yml file.
-
+        reflow (bool): if set to False, the original layout
+            (line endings, indentation) of the yml file
+            will be preserved (useful for files containing bullet lists etc.);
+            in the output string, new line characters will be replaced with ¶.
+            if set to True, new line characters will be removed
+            (except double line breaks and line breaks in bullet lists)
+            and the indentation and line length will be standardized.
+            
     Returns:
         (dict): dictionary representation of the yml key-value pairs
 
@@ -125,10 +132,16 @@ def readYML(fp):
 ##        {}
     """
     with open(fp, "r", encoding="utf8") as file:
-        return ymlToDic(file.read())
+        try:
+           return ymlToDic(file.read(), yml_fp=fp)
+        except Exception as e:
+           print(fp)
+           print(e)
 
 
-def dicToYML(dic, max_length=72, reflow=True):
+
+
+def dicToYML(dic, max_length=80, reflow=True, break_long_words=False):
     """Convert a dictionary into a yml string.
 
     NB: use the pilcrow (¶) to force a line break within dictionary values.
@@ -141,6 +154,8 @@ def dicToYML(dic, max_length=72, reflow=True):
             will be preserved (useful for files containing bullet lists etc.);
             if set to True, the indentation and line length
             will be standardized.
+        break_long_words (bool): if False, long words will be kept on one line
+            
     Returns:
         (str): yml string representation of the dic's key-value pairs
 
@@ -170,30 +185,89 @@ def dicToYML(dic, max_length=72, reflow=True):
         True
     """
     data = []
-    for k,v in dic.items():
-        if k.strip().endswith(":"):
-            i = "{} {}".format(k.strip(), str(v).strip())
-        else:
-            i = "{}: {}".format(k.strip(), str(v).strip())
+    if dic:
+        for k,v in dic.items():
+            if k.strip().endswith(":"):
+                i = "{} {}".format(k.strip(), str(v).strip())
+            else:
+                i = "{}: {}".format(k.strip(), str(v).strip())
 
-        # split long values into indented multiline values:
+            # split long values into indented multiline values:
 
-        if "#URI#" not in i:
-            lines = re.split("¶", i)
-            if len(lines) > 1:
-                lines = [lines[0]] + [line if line.startswith((" ", "\t")) else "    "+line
-                                      for line in lines[1:]]
-            
-            if reflow:
-                lines = ["\n    ".join(textwrap.wrap(line, max_length,
-                                                     break_long_words=False))
-                         for line in lines]
-            i = "\n".join(lines)
-        data.append(i)
+            if "#URI#" not in i:
+
+                lines = re.split("¶", i)
+
+                if len(lines) > 1:
+                    lines = [lines[0]] + [line if line.startswith((" ", "\t")) else "    "+line
+                                          for line in lines[1:]]
+                
+                if reflow:
+                    lines = ["\n    ".join(textwrap.wrap(line.strip(), max_length,
+                                                         break_long_words=break_long_words))
+                             for line in lines]
+
+                i = "\n    ".join(lines)
+            data.append(i)
 
     return "\n".join(sorted(data))
 
+def fix_broken_yml(fp, execute=True):
+    """Fix a yml file that is broken because
+    (1) a line does not start with a valid key or space
+    or (2) the colon after the key is absent
+
+    Args:
+        fp (str): path to the broken yml file
+        execute (bool): if False, user's judgment about the fix
+             will be asked before the fix is implemented
+
+    Returns:
+        None or yml_d
+    """
+    with open(fp, mode="r", encoding="utf-8") as file:
+        data = file.read()
+    key_lines = []
+    current = []
+    for line in data.splitlines():
+        if re.findall("^\d\d#[#\w]{14}:?", line):
+            if current:
+                key_lines.append(current)
+                current = []
+        current.append(line.strip())
+    if current:
+        key_lines.append(current)
+    key_lines = ["¶".join(line) for line in key_lines]
+    key_regex = "^(\d\d#[#\w]{14}):?"
+    yml_d = {re.findall(key_regex, line)[0]+":" : re.sub(key_regex, "", line).strip()
+             for line in key_lines}
+    print("original yml file:\n")
+    print(data)
+    print("\nAttempt to solve the issue:\n")
+    print(dicToYML(yml_d))
+    if execute or input("\nAccept change? Y/N: ").lower() == "y":
+        return yml_d
+    else:
+        print("Aborting change. Please review YML file manually")
+        return
+
+
 if __name__ == "__main__":
+    yml_fp = "../../../../../OpenITI/Github_clone/0875AH/data/0852IbnHajarCasqalani/0852IbnHajarCasqalani.InbaGhumr/0852IbnHajarCasqalani.InbaGhumr.Shamela0026317-ara1.yml" 
+#    with open(yml_fp, mode="r", encoding="utf-8") as file:
+#        t = file.read()
+#    print(t)
+#    input("continue?")
+#    t = t.replace("\xa0", " ")
+#    print(t)
+#    input("continue?")
+#    with open(yml_fp, mode="w", encoding="utf-8") as file:
+#        file.write(t)
+#    yml_dic = readYML(yml_fp)
+#    print(yml_dic)
+#    yml_str = dicToYML(yml_dic)
+#    print(yml_str)
+#    input("continue?")
     yml_str = """\
 key1: short description
 key2: longer description with : :: colons
@@ -205,7 +279,6 @@ key3: longer multiline description longer multiline description longer multiline
 
     * bullet item 1
     * bullet item 2
- 
 """
     print("do not reflow:")
     print(ymlToDic(yml_str, reflow=False))
@@ -224,4 +297,4 @@ key3: longer multiline description longer multiline description longer multiline
     import doctest
     input()
 
-    doctest.testmod()
+#    doctest.testmod()
